from flask_wtf import FlaskForm
from flask_login import current_user
from wtforms import StringField, SubmitField, TextAreaField, BooleanField, SelectField, DateField, IntegerField, FloatField, PasswordField
from wtforms.validators import Email,  ValidationError, DataRequired, Length, EqualTo, NumberRange, InputRequired, Regexp
from app.models import User, ROLES, PaymentType, Restaurant
from sqlalchemy import exc


class EditProfileForm(FlaskForm):
    username = TextAreaField('Username', validators=[Length(min=3, max=32)])
    email = TextAreaField('Email', validators=[Length(min=6, max=40), Email()])
    first_name = TextAreaField('First name', validators=[Length(min=1, max=32)])
    last_name = TextAreaField('Last name', validators=[Length(min=1, max=32)])
    submit = SubmitField('Submit')

    def __init__(self, original_username, *args, **kwargs):
        super(EditProfileForm, self).__init__(*args, **kwargs)
        self.original_username = original_username

    def validate_username(self, username):
        if username.data != self.original_username:
            user = User.query.filter_by(username=self.username.data).first()
            if user is not None:
                raise ValidationError('Please use a different username.')


class EditUserForm(FlaskForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.role.choices = [r[0] for r in ROLES]

    username = TextAreaField('Username', validators=[Length(min=3, max=32)])
    email = TextAreaField('Email', validators=[Length(min=6, max=40), Email()])
    first_name = TextAreaField('First name', validators=[Length(min=1, max=32)])
    last_name = TextAreaField('Last name', validators=[Length(min=1, max=32)])
    active = BooleanField("User is active?", default=False)
    is_disabled = BooleanField("Disable user?", default=False)
    role = SelectField("User role")

    submit = SubmitField('Submit')


class NewUserForm(FlaskForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.role.default = "driver"
        self.role.choices = [r[0] for r in ROLES]

    username = TextAreaField('Username', validators=[Length(min=3, max=32)])
    email = TextAreaField('Email', validators=[Length(min=6, max=40), Email()])
    password = PasswordField('Password', validators=[InputRequired()])
    password2 = PasswordField('Repeat Password', validators=[InputRequired(), EqualTo('password')])
    first_name = TextAreaField('First name', validators=[Length(min=1, max=32)])
    last_name = TextAreaField('Last name', validators=[Length(min=1, max=32)])
    role = SelectField("User role")

    submit = SubmitField('Submit')

    def validate_email(self, email):
        user = User.query.filter_by(email=self.email.data).first()
        if user is not None:
            raise ValidationError(f'There is another account already registered with email: {email.data}')


class EditOrderForm(FlaskForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        restaurants = Restaurant.query.all()
        payment_types = PaymentType.query.all()
        users = User.query.filter_by(role="driver").all()
        self.restaurant.choices = [r.name for r in restaurants]
        self.customer_payment_type.choices = [p.name for p in payment_types]
        self.user.choices = [u.username for u in users]

    user = SelectField("Driver")
    date = DateField("Date")
    restaurant = SelectField("Restaurant")
    receipt = StringField("Receipt #", validators=[InputRequired()])
    customer_address = StringField('Customer Address', validators=[InputRequired()])
    driver_paid_amount = StringField("Driver Paid", validators=[InputRequired(), Regexp(r"^[0-9]*\.?[0-9]{0,2}$")])
    total_restaurant_amount = StringField("Restaurant Total", validators=[InputRequired(), Regexp(r"^[0-9]*\.?[0-9]{0,2}$")])
    tips = StringField("Tips", validators=[InputRequired(), Regexp(r"^[0-9]*\.?[0-9]{0,2}$")])
    customer_payment_type = SelectField("Payment Type")
    additional_pickup = BooleanField("Additional Pickup?")
    last_order = BooleanField("Driver's last order of the day?", default=False)
    submit = SubmitField('Submit')


class EditRestaurantForm(FlaskForm):
    name = StringField('Restaurant Name', validators=[DataRequired()])
    driver_fee = FloatField("Portion of delivery fee that goes to the driver", validators=[InputRequired(), NumberRange(min=0)])
    additional_driver_fee = FloatField("Portion of the 'Additional Delivery Fee' that goes to the driver", validators=[InputRequired(), NumberRange(min=0)])
    crave_fee = FloatField("Portion of delivery fee that goes to Crave", validators=[InputRequired(), NumberRange(min=0)])
    additional_crave_fee = FloatField("Poriton of the 'Additional Delivery Fee' that goes to crave", validators=[InputRequired(), NumberRange(min=0)])
    service_fee = FloatField("Service Fee Percentage (write as a decimal, 0.05 not 5%)", validators=[InputRequired(), NumberRange(min=0)])
    commission = FloatField("Commission Percentage (write as a decimal, 0.05 not 5%)", validators=[InputRequired(), NumberRange(min=0)])
    prepaid_commission = FloatField("Food Discount Percentage for 'Pre-paid only' restaurant", validators=[InputRequired(), NumberRange(min=0)])
    monthly_fee = FloatField("Monthly Fee", validators=[InputRequired(), NumberRange(min=0)])
    delivery_rebate = FloatField("Per-delivery Rebate", validators=[InputRequired(), NumberRange(min=0)])

    prepaid_only = BooleanField("Restrict payment methods to 'pick-up only'?")
    active = BooleanField("Restaurant appears in drop-down lists?")

    submit = SubmitField('Submit')


class NewRestaurantForm(FlaskForm):
    name = StringField('Restaurant Name', validators=[InputRequired()])
    driver_fee = FloatField("Portion of delivery fee that goes to the driver", validators=[InputRequired(), NumberRange(min=0)], default=0)
    additional_driver_fee = FloatField("Portion of the 'Additional Delivery Fee' that goes to the driver", validators=[InputRequired(), NumberRange(min=0)], default=0)
    crave_fee = FloatField("Portion of delivery fee that goes to Crave", validators=[InputRequired(), NumberRange(min=0)], default=0)
    additional_crave_fee = FloatField("Poriton of the 'Additional Delivery Fee' that goes to crave", validators=[InputRequired(), NumberRange(min=0)], default=0)
    service_fee = FloatField("Service Fee Percentage (write as a decimal, 0.05 not 5%)", validators=[InputRequired(), NumberRange(min=0)], default=0)
    commission = FloatField("Commission Percentage (write as a decimal, 0.05 not 5%)", validators=[InputRequired(), NumberRange(min=0)], default=0)
    prepaid_commission = FloatField("Food Discount Percentage for 'Pre-paid only' restaurant", validators=[InputRequired(), NumberRange(min=0)], default=0)
    monthly_fee = FloatField("Monthly Fee", validators=[InputRequired(), NumberRange(min=0)])
    delivery_rebate = FloatField("Per-delivery Rebate", validators=[InputRequired(), NumberRange(min=0)])

    prepaid_only = BooleanField("Restrict payment methods to 'pre-paid only'?")
    active = BooleanField("Restaurant appears in drop-down lists?", default=True)

    submit = SubmitField('Submit')

    def validate_name(self, name):
        restaurant = Restaurant.query.filter_by(name=self.name.data).first()
        if restaurant is not None:
            raise ValidationError(f'There is already a restaurant with the name "{name.data}".\nPlease User another name or edit the existing restaurant.')


class AdminResetPasswordForm(FlaskForm):
    current_password = PasswordField('Current Password', validators=[DataRequired()])
    new_password = PasswordField('New Password', validators=[DataRequired(), EqualTo('password2', message='Passwords must match')])
    password2 = PasswordField('Confirm password', validators=[DataRequired()])
    submit = SubmitField('Reset Password')

    def validate_current_password(self, field):
        if not current_user.check_password(field.data):
            raise ValidationError('Current password is wrong')


class EmptyForm(FlaskForm):
    submit = SubmitField('Submit')

